#perl
#ver 0.3
#author : Seneca Menard

#setItemPosToVert : sets selected items' positions to be the avg position of the selected verts.  To set pos in only X,Y,Z, use those args.
#selectNewestByType : selects the newest item of a certain type, sorted by creation date.  The type is the arg.
#selectAllGradValues : if you have 20 grads selected, and want to select all the keys, run this.  (you'll need the "rgb" arg if you want to sel rgb instead of value.)
#moveScreen : moves selected items forwards or backwards in world space or screen space.  args=(forward,backward,large,small,worldAxis)
#rectifyItemXfrms : will go through all items and give them transforms, and on meshInstances, it will give them unique transforms so you won't get inadvertent transforms occurring in the future because of the stupid modo feature that creates transform links from meshes to their instances.

#This script has a number of item tools.  Right now there's only one.
foreach my $arg (@ARGV){
	if		($arg eq "X")					{	our $x = 1;				}
	elsif	($arg eq "Y")					{	our $y = 1;				}
	elsif	($arg eq "Z")					{	our $z = 1;				}
	elsif	($arg eq "set")					{	our $selMode = "set";	}
	elsif	($arg eq "add")					{	our $selMode = "add";	}
	elsif	($arg eq "small")				{	our $small = 1;			}
	elsif	($arg eq "large")				{	our $large = 1;			}
	elsif	($arg eq "worldAxis")			{	our $worldAxis = 1;		}
	elsif	($arg =~ "selNewTypeDlg")		{	our $selNewTypeDlg = 1;	}
	elsif	($arg =~ /setItemPosToVert/i)	{	setItemPosToVert();		}
	elsif	($arg =~ /selectNewest/i)		{	selectNewestByType();	}
	elsif	($arg =~ /selectAllGrad/i)		{	selectAllGradValues();	}
	elsif	($arg =~ /moveScreen/i)			{	moveScreen();			}
	elsif	($arg =~ /moveToRagePos/i)		{	moveToRagePos();		}
	elsif	($arg =~ /rectifyItemXfrms/i)	{	rectifyItemXfrms();		}
	elsif	($arg =~ /replaceInstances/i)	{	replaceInstances();		}
	elsif	($arg =~ /cnvInstToMeshWPart/i)	{	cnvInstToMeshWPart();	}
	elsif	($arg =~ /delEmptyMshsNGrps/i)	{	delEmptyMshsNGrps();	}
	else									{	our $miscArg = $arg;	}
}

#deletes empty meshes and groups.  (if mesh has children, won't delete so youd have to run it twice)
sub delEmptyMshsNGrps{
	lx("!!select.drop item");
	lx("!!select.itemType mesh");
	my @meshSel = lxq("query sceneservice selection ? mesh");

	foreach my $id (@meshSel){
		my $layerName = lxq("query layerservice layer.name ? {$id}");
		my $vertCount = lxq("query layerservice vert.n ? all");
		if ($vertCount == 0){
			my $childCount = lxq("query sceneservice item.numChildren ? {$id}");
			if ($childCount == 0){
				lx("select.item {$id} set");
				lx("item.delete");
			}else{
				lxout("--couldn't delete '$layerName' ($id) because it had children");
			}
		}
	}

	lx("!!select.drop item");
	lx("!!select.itemType groupLocator");
	my @groupSel = lxq("query sceneservice selection ? groupLocator");

	foreach my $id (@groupSel){
		my $childCount = lxq("query sceneservice item.numChildren ? {$id}");
		if ($childCount == 0){
			lx("select.item {$id} set") or next;
			lx("item.delete");
		}else{
			my $name = lxq("query sceneservice item.name ? {$id}");
			lxout("--couldn't delete '$name' ($id) because it had children");
		}
	}
}


#convert meshes to instances and apply a random part name to each.
sub cnvInstToMeshWPart{
	srand;
	my %itemTypes;
		$itemTypes{"mesh"} = 1;
		$itemTypes{"meshInst"} = 1;

	my @selection = lxq("query sceneservice selection ? all");
	lxout("selection = @selection");

	foreach my $id (@selection){
		if		(lxq("query sceneservice item.type ? {$id}") eq "mesh"){
			lxout("sdf");
			lx("select.item {$id} set");
		}elsif	(lxq("query sceneservice item.type ? {$id}") eq "meshInst"){
			lxout("sdf2");
			lx("select.item {$id} set");
			lx("item.setType mesh locator");
		}

		my $partName = "";
		my @alphabet = (0,1,2,3,4,5,6,7,8,9,0,"a","b","c","d","e","f","g","h","i","j","k","l","m","n","o","p","q","r","s","t","u","v","w","x","y","z");
		for (my $i=0; $i<12; $i++){$partName .= @alphabet[rand(35)];}
		lxout("partName = $partName");
		lx("!!select.type polygon");
		lx("!!poly.setPart {$partName}");
	}
}


#ignoring pivot, rot order and mult transforms. (piv queries fail, the others are just laziness)
sub replaceInstances{
	my %itemTypes;
		$itemTypes{"mesh"} = 1;
		$itemTypes{"meshInst"} = 1;

	my @selection = lxq("query sceneservice selection ? all");
	my $foundMeshID = "";

	#find first selected mesh
	foreach my $id (@selection){
		if		(lxq("query sceneservice item.type ? {$id}") eq "mesh"){
			$foundMeshID = $id;
			last;
		}elsif	(lxq("query sceneservice item.type ? {$id}") eq "meshInst"){
			$foundMeshID = lxq("query sceneservice item.source ? {$id}");
			last;
		}
	}

	#go through all other selected meshes and replace them with first found mesh.
	my $counter = 0;
	foreach my $id (@selection){
		if ($itemTypes{lxq("query sceneservice item.type ? {$id}")} == 1){
			if ($counter > 0){

				if (lxq("query sceneservice item.type ? {$id}") eq "mesh"){
					my $name = lxq("query sceneservice item.name ? {$id}");
					if (quickDialog("($name) is a mesh! Replace?","yesNo","","","","") =~ /no/i){next;}
				}

				my $parent = lxq("query sceneservice item.parent ? {$id}");
				my $posXfrmID = lxq("query sceneservice item.xfrmPos ? {$id}");
				my $rotXfrmID = lxq("query sceneservice item.xfrmRot ? {$id}");
				my $sclXfrmID = lxq("query sceneservice item.xfrmScl ? {$id}");

				#create transforms if missing
				if ($posXfrmID eq ""){lx("transform.add type:{pos} item:{$id}");}
				if ($rotXfrmID eq ""){lx("transform.add type:{rot} item:{$id}");}
				if ($sclXfrmID eq ""){lx("transform.add type:{scl} item:{$id}");}

				#query transforms
				my $xPos = lxq("item.channel pos.X {?} set {$id}");
				my $yPos = lxq("item.channel pos.Y {?} set {$id}");
				my $zPos = lxq("item.channel pos.Z {?} set {$id}");
				my $xRot = lxq("item.channel rot.X {?} set {$id}");
				my $yRot = lxq("item.channel rot.Y {?} set {$id}");
				my $zRot = lxq("item.channel rot.Z {?} set {$id}");
				my $xScl = lxq("item.channel scl.X {?} set {$id}");
				my $yScl = lxq("item.channel scl.Y {?} set {$id}");
				my $zScl = lxq("item.channel scl.Z {?} set {$id}");

				#dupe original item and give back transforms and parent
				lx("select.item {$foundMeshID} set");
				lx("item.duplicate true locator");
				my @meshInsts = lxq("query sceneservice selection ? meshInst");

				my $newPosXfrmID = lxq("query sceneservice item.xfrmPos ? {$meshInsts[0]}");
				my $newRotXfrmID = lxq("query sceneservice item.xfrmRot ? {$meshInsts[0]}");
				my $newSclXfrmID = lxq("query sceneservice item.xfrmScl ? {$meshInsts[0]}");

				if ($newPosXfrmID eq ""){lx("transform.add type:{pos} item:{$meshInsts[0]}");}
				if ($newRotXfrmID eq ""){lx("transform.add type:{rot} item:{$meshInsts[0]}");}
				if ($newSclXfrmID eq ""){lx("transform.add type:{scl} item:{$meshInsts[0]}");}

				lx("item.channel pos.X {$xPos} set {$meshInsts[0]}");
				lx("item.channel pos.Y {$yPos} set {$meshInsts[0]}");
				lx("item.channel pos.Z {$zPos} set {$meshInsts[0]}");
				lx("item.channel rot.X {$xRot} set {$meshInsts[0]}");
				lx("item.channel rot.Y {$yRot} set {$meshInsts[0]}");
				lx("item.channel rot.Z {$zRot} set {$meshInsts[0]}");
				lx("item.channel scl.X {$xScl} set {$meshInsts[0]}");
				lx("item.channel scl.Y {$yScl} set {$meshInsts[0]}");
				lx("item.channel scl.Z {$zScl} set {$meshInsts[0]}");
				if ($parent ne ""){lx("item.parent item:{$meshInsts[0]} parent:{$parent} inPlace:{0}");}

				#delete old item.
				lx("select.item {$id} set");
				lx("item.delete");
			}
			$counter++;
		}
	}

	lx("select.item {$foundMeshID} set");
}


sub rectifyItemXfrms{
	my %itemTypes;
		$itemTypes{"mesh"} = 1;
		$itemTypes{"meshInst"} = 1;
		$itemTypes{"groupLocator"} = 1;

	#force all items to have pos,rot,scl channels
	my @selection = lxq("query sceneservice selection ? all");
	foreach my $id (@selection){
		if ($itemTypes{lxq("query sceneservice item.type ? $id")} == 1){
			if (lxq("query sceneservice item.xfrmPos ? $id") eq "")	{	lxout("eh? pos");	lx("transform.add type:pos item:{$id}");	}
			if (lxq("query sceneservice item.xfrmRot ? $id") eq "")	{	lxout("eh? rot");	lx("transform.add type:rot item:{$id}");	}
			if (lxq("query sceneservice item.xfrmScl ? $id") eq "")	{	lxout("eh? scl");	lx("transform.add type:scl item:{$id}");	}
		}
	}

	#force overwrite all translation channels in each instance
	foreach my $id (@selection){
		if (lxq("query sceneservice item.type ? $id") eq "meshInst"){
			my $itemName = lxq("query sceneservice item.name ? $id");
			my $id_xfrmPos = lxq("query sceneservice item.xfrmPos ? $id");
			my $id_xfrmRot = lxq("query sceneservice item.xfrmRot ? $id");
			my $id_xfrmScl = lxq("query sceneservice item.xfrmScl ? $id");

			my $posX = lxq("item.channel pos.X {?} set {$id_xfrmPos}");
			my $posY = lxq("item.channel pos.Y {?} set {$id_xfrmPos}");
			my $posZ = lxq("item.channel pos.Z {?} set {$id_xfrmPos}");

			my $rotX = lxq("item.channel rot.X {?} set {$id_xfrmRot}");
			my $rotY = lxq("item.channel rot.Y {?} set {$id_xfrmRot}");
			my $rotZ = lxq("item.channel rot.Z {?} set {$id_xfrmRot}");

			my $sclX = lxq("item.channel scl.X {?} set {$id_xfrmScl}");
			my $sclY = lxq("item.channel scl.Y {?} set {$id_xfrmScl}");
			my $sclZ = lxq("item.channel scl.Z {?} set {$id_xfrmScl}");

			my $posXTemp = $posX + 1;
			my $posYTemp = $posY + 1;
			my $posZTemp = $posZ + 1;

			my $rotXTemp = $rotX + 1;
			my $rotYTemp = $rotY + 1;
			my $rotZTemp = $rotZ + 1;

			my $sclXTemp = $sclX + 1;
			my $sclYTemp = $sclY + 1;
			my $sclZTemp = $sclZ + 1;

			lx("item.channel pos.X {$posXTemp} set {$id_xfrmPos}");
			lx("item.channel pos.Y {$posYTemp} set {$id_xfrmPos}");
			lx("item.channel pos.Z {$posZTemp} set {$id_xfrmPos}");

			lx("item.channel rot.X {$rotXTemp} set {$id_xfrmRot}");
			lx("item.channel rot.Y {$rotYTemp} set {$id_xfrmRot}");
			lx("item.channel rot.Z {$rotZTemp} set {$id_xfrmRot}");

			lx("item.channel scl.X {$sclXTemp} set {$id_xfrmScl}");
			lx("item.channel scl.Y {$sclYTemp} set {$id_xfrmScl}");
			lx("item.channel scl.Z {$sclZTemp} set {$id_xfrmScl}");

			lx("item.channel pos.X {$posX} set {$id_xfrmPos}");
			lx("item.channel pos.Y {$posY} set {$id_xfrmPos}");
			lx("item.channel pos.Z {$posZ} set {$id_xfrmPos}");

			lx("item.channel rot.X {$rotX} set {$id_xfrmRot}");
			lx("item.channel rot.Y {$rotY} set {$id_xfrmRot}");
			lx("item.channel rot.Z {$rotZ} set {$id_xfrmRot}");

			lx("item.channel scl.X {$sclX} set {$id_xfrmScl}");
			lx("item.channel scl.Y {$sclY} set {$id_xfrmScl}");
			lx("item.channel scl.Z {$sclZ} set {$id_xfrmScl}");
		}
	}
}

sub moveToRagePos{
	my %itemFilterTable;
		$itemFilterTable{mesh} = 1;
		$itemFilterTable{meshInst} = 1;
		$itemFilterTable{triSurf} = 1;
		$itemFilterTable{groupLocator} = 1;
		$itemFilterTable{locator} = 1;
		$itemFilterTable{light} = 1;
		$itemFilterTable{sunLight} = 1;
		$itemFilterTable{camera} = 1;


	my @selection = lxq("query sceneservice selection ? all");
	my $text = quickDialog("type in the idstudio position:",string,"100,200,300","","");
	my @pos = split(/,/, $text);
	my @newPos = ($pos[0], $pos[2], $pos[1] * -1);

	foreach my $item (@selection){
		my $type = lxq("query sceneservice item.type ? $item");
		if ($itemFilterTable{$type} == 1){
			my $xfrmPosID = lxq("query sceneservice item.xfrmPos ? $item");
			if ($xfrmPosID eq ""){lx("transform.add type:pos item:$item");}

			lx("item.channel pos.X {$newPos[0]} set {$item}");
			lx("item.channel pos.Y {$newPos[1]} set {$item}");
			lx("item.channel pos.Z {$newPos[2]} set {$item}");
		}
	}
}

sub moveScreen{
	my %itemFilterTable;
		$itemFilterTable{mesh} = 1;
		$itemFilterTable{meshInst} = 1;
		$itemFilterTable{triSurf} = 1;
		$itemFilterTable{groupLocator} = 1;
		$itemFilterTable{locator} = 1;
		$itemFilterTable{light} = 1;
		$itemFilterTable{sunLight} = 1;
		$itemFilterTable{camera} = 1;

	#find the grid numeric system
	if (lxq("pref.value units.system ?") eq "game")	{	our $power = 2;		}
	else											{	our $power = 10;	}

	#find viewport dir
	my $viewport = lxq("query view3dservice mouse.view ?");
	my @viewAxis = lxq("query view3dservice view.axis ? $viewport");
	my @frame = lxq("query view3dservice view.frame ?");
	my $scale = lxq("query view3dservice view.scale ? $viewport");
	my @viewBbox = lxq("query view3dservice view.rect ? $viewport");
	my @viewScale = (($scale * @viewBbox[2]),($scale * @viewBbox[3]));
	my @div = (($viewScale[0]/16),($viewScale[1]/16));
	my $log = int(.65 + (log(@div[0])/log($power)));
	my $grid = $power ** $log;

	#=============================
	#world axis
	#=============================

	#find primary axis
	my @xAxis = (1,0,0);
	my @yAxis = (0,1,0);
	my @zAxis = (0,0,1);
	my $dp0 = dotProduct(\@viewAxis,\@xAxis);
	my $dp1 = dotProduct(\@viewAxis,\@yAxis);
	my $dp2 = dotProduct(\@viewAxis,\@zAxis);
	if 		((abs($dp0) >= abs($dp1)) && (abs($dp0) >= abs($dp2)))	{	our $axis = 0;	lxout("[->] : Using world X axis");}
	elsif	((abs($dp1) >= abs($dp0)) && (abs($dp1) >= abs($dp2)))	{	our $axis = 1;	lxout("[->] : Using world Y axis");}
	else															{	our $axis = 2;	lxout("[->] : Using world Z axis");}

	#flip move pos if cam facing away from positive axis
	my @posChannelName = ("pos.X","pos.Y","pos.Z");
	if		($axis == 0)	{	our $axisViewDP = dotProduct(\@viewAxis,\@xAxis);	}
	elsif	($axis == 1)	{	our $axisViewDP = dotProduct(\@viewAxis,\@xAxis);	}
	elsif	($axis == 2)	{	our $axisViewDP = dotProduct(\@viewAxis,\@xAxis);	}
	if ($axisViewDP < 0)	{	$axisViewDP = -1;									}
	else					{	$axisViewDP = 1;									}

	#determine move amount scalar
	if		($small == 1)	{	our $moveScalar = .25;	}
	elsif	($large == 1)	{	our $moveScalar = 4;	}
	else					{	our $moveScalar = 1;	}

	my @selection = lxq("query sceneservice selection ? all");
	foreach my $id (@selection){
		my $type = lxq("query sceneservice item.type ? $id");
		if ($itemFilterTable{$type} == 1){
			my $xfrmPosID = lxq("query sceneservice item.xfrmPos ? $id");
			if ($xfrmPosID eq ""){lx("transform.add type:pos item:$id");}

			#move in world space
			if ($worldAxis == 1){
				my $axisPos = lxq("item.channel {$posChannelName[$axis]} {?} set {$id}");
				if ($miscArg eq "forward")	{	$axisPos += ($grid * $axisViewDP * $moveScalar);	}
				else						{	$axisPos -= ($grid * $axisViewDP * $moveScalar);	}
				lx("item.channel {$posChannelName[$axis]} {$axisPos} set {$id}");
			}
			#move in screen space
			else{
				my $axisPosX = lxq("item.channel pos.X {?} set {$id}");
				my $axisPosY = lxq("item.channel pos.Y {?} set {$id}");
				my $axisPosZ = lxq("item.channel pos.Z {?} set {$id}");

				if ($miscArg eq "forward"){
					our $newPosX = $axisPosX + ($viewAxis[0] * $grid * $moveScalar);
					our $newPosY = $axisPosY + ($viewAxis[1] * $grid * $moveScalar);
					our $newPosZ = $axisPosZ + ($viewAxis[2] * $grid * $moveScalar);
				}else{
					our $newPosX = $axisPosX - ($viewAxis[0] * $grid * $moveScalar);
					our $newPosY = $axisPosY - ($viewAxis[1] * $grid * $moveScalar);
					our $newPosZ = $axisPosZ - ($viewAxis[2] * $grid * $moveScalar);
				}

				lx("item.channel pos.X {$newPosX} set {$id}");
				lx("item.channel pos.Y {$newPosY} set {$id}");
				lx("item.channel pos.Z {$newPosZ} set {$id}");
			}
		}
	}
}

#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#SELECT ALL GRADATION VALUES
#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
sub selectAllGradValues{
	my @gradationSelection = lxq("query sceneservice selection ? gradient");
	foreach my $id (@gradationSelection){
		if ($miscArg =~ /rgb/i){
			lx("select.channel {$id:color.R} add");
			lx("select.channel {$id:color.G} add");
			lx("select.channel {$id:color.B} add");
		}else{
			lx("select.channel {$id:value} add");
		}
	}
}

#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#SELECT NEWEST ITEM BY TYPE
#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
sub selectNewestByType{
	if ($selNewTypeDlg == 1){
		$itemTypesList = "mesh;meshInst;triSurf;groupLocator;locator;camera;sunLight;pointLight;spotLight;photometryLight;domeLight;cylinderLight;areaLight";
		our $miscArg = popupMultChoice("Item type:",$itemTypesList,0);
	}
	if ($selMode eq ""){$selMode = "set";}
	my $itemCount = lxq("query sceneservice item.n ? all");
	for (my $i=$itemCount-1; $i>-1; $i--){
		my $type = lxq("query sceneservice item.type ? $i");
		if (lxq("query sceneservice item.type ? $i") eq $miscArg){
			my $id = lxq("query sceneservice item.id ? $i");
			lx("select.item item:$id mode:$selMode");
			last;
		}
	}
}

#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#SET ITEM POS TO VERT POS SUB
#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
sub setItemPosToVert{
	lxout("[->] : Moving selected items to selected vert(s) position");
	my @selection = lxq("query sceneservice selection ? all");
	my $mainlayer = lxq("query layerservice layers ? main");
	my @verts = lxq("query layerservice verts ? selected");
	if (@verts == 0){die("You don't have any verts selected and so I'm cancelling the script");}
	my @pos = (0,0,0);
	my %itemFilterTable;
		$itemFilterTable{mesh} = 1;
		$itemFilterTable{meshInst} = 1;
		$itemFilterTable{triSurf} = 1;
		$itemFilterTable{groupLocator} = 1;
		$itemFilterTable{locator} = 1;
		$itemFilterTable{light} = 1;
		$itemFilterTable{sunLight} = 1;
		$itemFilterTable{camera} = 1;

	foreach my $vert (@verts){
		my @vertPos = lxq("query layerservice vert.pos ? $vert");
		@pos = arrMath(@pos,@vertPos,add);
	}
	@pos = arrMath(@pos,$#verts+1,$#verts+1,$#verts+1,div);

	foreach my $id (@selection){
		my $type = lxq("query sceneservice item.type ? $id");
		if ($itemFilterTable{$type} == 1){
			my $xfrmPosID = lxq("query sceneservice item.xfrmPos ? $id");
			if ($xfrmPosID eq ""){lx("transform.add type:pos item:$id");}

			if (($x == 1) || ($y == 1) || ($z == 1)){
				if ($x == 1)	{	lx("item.channel pos.X {$pos[0]} set {$id}");	}
				if ($y == 1)	{	lx("item.channel pos.Y {$pos[1]} set {$id}");	}
				if ($z == 1)	{	lx("item.channel pos.Z {$pos[2]} set {$id}");	}
			}else{
				lx("item.channel pos.X {$pos[0]} set {$id}");
				lx("item.channel pos.Y {$pos[1]} set {$id}");
				lx("item.channel pos.Z {$pos[2]} set {$id}");
			}
		}
	}
}



#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#PERFORM MATH FROM ONE ARRAY TO ANOTHER subroutine
#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#USAGE : my @disp = arrMath(@pos2,@pos1,subt);
sub arrMath{
	my @array1 = (@_[0],@_[1],@_[2]);
	my @array2 = (@_[3],@_[4],@_[5]);
	my $math = @_[6];

	my @newArray;
	if		($math eq "add")	{	@newArray = (@array1[0]+@array2[0],@array1[1]+@array2[1],@array1[2]+@array2[2]);	}
	elsif	($math eq "subt")	{	@newArray = (@array1[0]-@array2[0],@array1[1]-@array2[1],@array1[2]-@array2[2]);	}
	elsif	($math eq "mult")	{	@newArray = (@array1[0]*@array2[0],@array1[1]*@array2[1],@array1[2]*@array2[2]);	}
	elsif	($math eq "div")	{	@newArray = (@array1[0]/@array2[0],@array1[1]/@array2[1],@array1[2]/@array2[2]);	}
	return @newArray;
}


#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#UNIT VECTOR SUBROUTINE
#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#USAGE : my @unitVector = unitVector(@vector);
sub unitVector{
	my $dist1 = sqrt((@_[0]*@_[0])+(@_[1]*@_[1])+(@_[2]*@_[2]));
	@_ = ((@_[0]/$dist1),(@_[1]/$dist1),(@_[2]/$dist1));
	return @_;
}

#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#DOT PRODUCT subroutine
#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#USAGE : my $dp = dotProduct(\@vector1,\@vector2);
sub dotProduct{
	my @array1 = @{$_[0]};
	my @array2 = @{$_[1]};
	my $dp = (	(@array1[0]*@array2[0])+(@array1[1]*@array2[1])+(@array1[2]*@array2[2])	);
	return $dp;
}

#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#QUICK DIALOG SUB (modded to not die if you click no in the yesNo routine)
#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#USAGE : quickDialog(username,float,initialValue,min,max);
sub quickDialog{
	if (@_[1] eq "yesNo"){
		lx("dialog.setup yesNo");
		lx("dialog.msg {@_[0]}");
		lx("dialog.open");
		return (lxq("dialog.result ?"));
	}else{
		if (lxq("query scriptsysservice userValue.isdefined ? seneTempDialog") == 0){
			lxout("-The seneTempDialog cvar didn't exist so I just created one");
			lx("user.defNew name:[seneTempDialog] life:[temporary]");
		}
		lx("user.def seneTempDialog username [@_[0]]");
		lx("user.def seneTempDialog type [@_[1]]");
		if ((@_[3] != "") && (@_[4] != "")){
			lx("user.def seneTempDialog min [@_[3]]");
			lx("user.def seneTempDialog max [@_[4]]");
		}
		lx("user.value seneTempDialog [@_[2]]");
		lx("user.value seneTempDialog ?");
		if (lxres != 0){	die("The user hit the cancel button");	}
		return(lxq("user.value seneTempDialog ?"));
	}
}

#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#POPUP SUB
#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#USAGE : popup("What I wanna print");
sub popup #(MODO2 FIX)
{
	lx("dialog.setup yesNo");
	lx("dialog.msg {@_}");
	lx("dialog.open");
	my $confirm = lxq("dialog.result ?");
	if($confirm eq "no"){die;}
}

##------------------------------------------------------------------------------------------------------------
##------------------------------------------------------------------------------------------------------------
#POPUP MULTIPLE CHOICE
##------------------------------------------------------------------------------------------------------------
##------------------------------------------------------------------------------------------------------------
#USAGE : my $answer = popupMultChoice("question name","yes;no;maybe;blahblah",$defaultChoiceInt);
sub popupMultChoice{
	if (lxq("query scriptsysservice userValue.isdefined ? seneTempDialog") == 1){lx("user.defDelete {seneTempDialog}");	}
	lx("user.defNew name:[seneTempDialog] type:[integer] life:[temporary]");
	lx("user.def seneTempDialog username [$_[0]]");
	lx("user.def seneTempDialog list {$_[1]}");
	lx("user.value seneTempDialog {$_[2]}");

	lx("user.value seneTempDialog");
	if (lxres != 0){	die("The user hit the cancel button");	}
	return(lxq("user.value seneTempDialog ?"));
}