#perl
#AUTHOR: Seneca Menard
#version 1.21 #NOTE : The polyNormal routine doesn't work yet because of bad matrix->eulers
#This script is to perform a poly slice across the screen using the viewport's axis, and the angle you defined by selecting (2 verts) or (1 edge).
#OPTIONS :
# : "fullCut" : If you want the knife cut to extend beyond the bounds of your currently selected "angle", append this cvar and the knife cut will be expanded until it's a little bit bigger than the current viewport (so it should always make a knife cut that's big enough).
# : "worldAxis" : If you don't want to use the screen axis, but actually want to use one of the 3 world axes, append this cvar.  You don't have to choose which world axis, the script will do it automatically for you based off of which axis you're looking down the most.
# : "edgeAxis" : Use this cvar to cut your polys by the chosen edge's edge normal.

#here's an example of what you bind a key to if you want to have it perform a cut that will cover the screen, using the world axis.  "@knifeScreen.pl fullCut worldAxis"

#INSTALLATION INSTRUCTIONS :
#1) : copy the icons dir to your scripts dir.  (ie, in windows it's here : C:\Documents and Settings\{USERNAME}\Application Data\Luxology\Scripts\icons)
#2) : copy the .CFG and .PL files to your scripts dir : (C:\Documents and Settings\{USERNAME}\Application Data\Luxology\Scripts)
#3) : open modo and bind a hotkey to the sen_KnifeScreen form and you're all set.

#SPECIAL NOTE :
#this script queries which 3d viewport is under the mouse and so if you bind this script to a button that's outside the 3d modeling windows, the script will not correctly guess which window was the last active 3d window and so you won't get the cut you were looking for..

#(8-1-08 bugfix) : I removed all the [] chars and swapped them for {} chars to get around the measurement system issue
#(4-23-10 feature) : edgeAxis cut is added, so you can have some polys selected and select an edge and it will cut the polys along the edge's normal.
#(6-29-10 bugfix) : reordered the item ref application to stop the viewport from being moved.
my $mainlayer = lxq("query layerservice layers ? main");
my $pi = 3.14159265358979323;
my $selType;

#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#---------------------------------------------------ARGS-------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
foreach my $arg (@ARGV){
	if		($arg =~ /fullCut/i)	{our $fullCut = 1;		}
	elsif	($arg =~ /worldAxis/i)	{our $worldAxis = 1;	}
	elsif	($arg =~ /vertical/i)	{our $vertical = 1;		}
	elsif	($arg =~ /horizontal/i)	{our $horizontal = 1;	}
	elsif	($arg =~ /edgeAxis/i)	{our $edgeAxis = 1;		}
	elsif	($arg =~ /polyNormal/i)	{our $polyNormal = 1;	}
}


#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#--------------------------------------SAFETY CHECKS---------------------------------------------
#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------

#symm
our $symmAxis = lxq("select.symmetryState ?");
if 		($symmAxis eq "none")	{	$symmAxis = 3;	}
elsif	($symmAxis eq "x")		{	$symmAxis = 0;	}
elsif	($symmAxis eq "y")		{	$symmAxis = 1;	}
elsif	($symmAxis eq "z")		{	$symmAxis = 2;	}
if ($symmAxis != 3){
	lx("select.symmetryState none");
}

#Remember what the workplane was and turn it off
my @WPmem;
@WPmem[0] = lxq ("workPlane.edit cenX:? ");
@WPmem[1] = lxq ("workPlane.edit cenY:? ");
@WPmem[2] = lxq ("workPlane.edit cenZ:? ");
@WPmem[3] = lxq ("workPlane.edit rotX:? ");
@WPmem[4] = lxq ("workPlane.edit rotY:? ");
@WPmem[5] = lxq ("workPlane.edit rotZ:? ");
lx("!!workPlane.reset");

#set the main layer to be "reference" to get the true vert positions.
my $mainlayerID = lxq("query layerservice layer.id ? $mainlayer");
my $layerReference = lxq("layer.setReference ?");
lx("!!layer.setReference $mainlayerID");

#-----------------------------------------------------------------------------------
#REMEMBER SELECTION SETTINGS   ((MODO2 FIX))  (modded : removed the selectauto setting)
#-----------------------------------------------------------------------------------
#sets the ACTR preset
my $seltype;
my $selAxis;
my $selCenter;
my $actr = 1;
if( lxq( "tool.set actr.select ?") eq "on")				{	$seltype = "actr.select";		}
elsif( lxq( "tool.set actr.selectauto ?") eq "on")		{	$seltype = "actr.selectauto";	}
elsif( lxq( "tool.set actr.element ?") eq "on")			{	$seltype = "actr.element";		}
elsif( lxq( "tool.set actr.screen ?") eq "on")			{	$seltype = "actr.screen";		}
elsif( lxq( "tool.set actr.origin ?") eq "on")			{	$seltype = "actr.origin";		}
elsif( lxq( "tool.set actr.local ?") eq "on")			{	$seltype = "actr.local";		}
elsif( lxq( "tool.set actr.pivot ?") eq "on")			{	$seltype = "actr.pivot";		}
elsif( lxq( "tool.set actr.auto ?") eq "on")			{	$seltype = "actr.auto";			}
else
{
	$actr = 0;
	lxout("custom Action Center");
	if( lxq( "tool.set axis.select ?") eq "on")			{	 $selAxis = "select";			}
	elsif( lxq( "tool.set axis.element ?") eq "on")		{	 $selAxis = "element";			}
	elsif( lxq( "tool.set axis.view ?") eq "on")		{	 $selAxis = "view";				}
	elsif( lxq( "tool.set axis.origin ?") eq "on")		{	 $selAxis = "origin";			}
	elsif( lxq( "tool.set axis.local ?") eq "on")		{	 $selAxis = "local";			}
	elsif( lxq( "tool.set axis.pivot ?") eq "on")		{	 $selAxis = "pivot";			}
	elsif( lxq( "tool.set axis.auto ?") eq "on")		{	 $selAxis = "auto";				}
	else												{	 $actr = 1;  $seltype = "actr.auto"; lxout("You were using an action AXIS that I couldn't read");}

	if( lxq( "tool.set center.select ?") eq "on")		{	 $selCenter = "select";			}
	elsif( lxq( "tool.set center.element ?") eq "on")	{	 $selCenter = "element";		}
	elsif( lxq( "tool.set center.view ?") eq "on")		{	 $selCenter = "view";			}
	elsif( lxq( "tool.set center.origin ?") eq "on")	{	 $selCenter = "origin";			}
	elsif( lxq( "tool.set center.local ?") eq "on")		{	 $selCenter = "local";			}
	elsif( lxq( "tool.set center.pivot ?") eq "on")		{	 $selCenter = "pivot";			}
	elsif( lxq( "tool.set center.auto ?") eq "on")		{	 $selCenter = "auto";			}
	else												{ 	 $actr = 1;  $seltype = "actr.auto"; lxout("You were using an action CENTER that I couldn't read");}
}



#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#------------------------------------------MAIN ROUTINE---------------------------------------------
#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------

#determine the axis
my $viewport = lxq("query view3dservice mouse.view ?");
my @axis = lxq("query view3dservice view.axis ? $viewport");
if ($worldAxis == 1){
	our $axis;
	my @xAxis = (1,0,0);
	my @yAxis = (0,1,0);
	my @zAxis = (0,0,1);
	my $dp0 = dotProduct(\@axis,\@xAxis);
	my $dp1 = dotProduct(\@axis,\@yAxis);
	my $dp2 = dotProduct(\@axis,\@zAxis);
	if 		((abs($dp0) >= abs($dp1)) && (abs($dp0) >= abs($dp2)))	{	$axis = 0;	lxout("[->] : Using world X axis");}
	elsif	((abs($dp1) >= abs($dp0)) && (abs($dp1) >= abs($dp2)))	{	$axis = 1;	lxout("[->] : Using world Y axis");}
	else															{	$axis = 2;	lxout("[->] : Using world Z axis");}
	lx("tool.set actr.auto on");
}elsif ($edgeAxis == 1){
	if (lxq("select.count edge ?") == 0){&die;}
	our @edges = lxq("query layerservice edges ? selected");

	if ($polyNormal == 1){
		my @polys = lxq("query layerservice edge.polyList ? $edges[-1]");
		my @edgeCenter = lxq("query layerservice edge.pos ? $edges[-1]");
		my $polysIndice = -1;
		my @edgeVerts;

		my $edge = $edges[-1];
		$edge =~ s/[()]//g;
		@verts = split(/,/, $edge);
		my @edgeVertPos1 = lxq("query layerservice vert.pos ? $verts[0]");

		if (@polys == 1){
			$polysIndice = 0;
		}else{
			my @normal1 = lxq("query layerservice poly.normal ? $polys[0]");
			my @normal2 = lxq("query layerservice poly.normal ? $polys[1]");
			my $dp1 = dotProduct(\@axis,\@normal1);
			my $dp2 = dotProduct(\@axis,\@normal2);
			if (abs($dp1) > abs($dp2))	{$polysIndice = 0;}
			else						{$polysIndice = 1;}
		}


		my @xVec = unitVector(arrMath(@edgeCenter,@edgeVertPos1,subt));
		my @yVec = lxq("query layerservice poly.normal ? $polys[$polysIndice]");
		my @zVec = unitVector(crossProduct(\@yVec,\@xVec));

		my @rotations = matrixToEuler(\@xVec,\@yVec,\@zVec);
		@rotations = (($rotations[0]*180)/$pi,($rotations[1]*180)/$pi,($rotations[2]*180)/$pi);

		lx("workPlane.edit {$edgeCenter[0]} {$edgeCenter[1]} {$edgeCenter[2]} {$rotations[0]} {$rotations[1]} {$rotations[2]}");
		popup("FAIL! The damn matrix to euler rotations conversion is failing because of bad rot order, so don't use this yet!");

	}else{
		lx("select.type edge");
		lx("workPlane.fitSelect");
	}
	lx("tool.set actr.auto on");
	our $axis = 1;
}else{
	our $axis = -1;
	lx("tool.set actr.screen on");
	lxout("[->] : Using SCREEN axis");
}

if ($edgeAxis == 1){
	my $edgeLength = lxq("query layerservice edge.length ? $edges[-1]");
	our @pos1 = ($edgeLength * -0.5 , 0 , 0);
	our @pos2 = ($edgeLength * 0.5 , 0 , 0);
	$selType = "polygon";
}else{
	if((lxq( "select.typeFrom {edge;polygon;item;vertex} ?" )) || ( lxq( "select.typeFrom {polygon;item;vertex;edge} ?" ))){
		if(lxq( "select.typeFrom {edge;polygon;item;vertex} ?" ))		{	$selType = "edge";	}else{	$selType = "polygon";	}
		if (lxq("query layerservice edge.n ? selected") > 0)												{	&getEdgePos;	}
		elsif ((($vertical == 1) || ($horizontal == 1)) && lxq("query layerservice vert.n ? selected") > 0)	{	&getVertPos;	}
		elsif (lxq("query layerservice vert.n ? selected") > 1)												{	&getVertPos;	}
		else																								{	&die;			}
	}
	elsif ( lxq( "select.typeFrom {vertex;edge;polygon;item} ?" ) ){
		$selType = "vertex";
		if ((($vertical == 1) || ($horizontal == 1)) && lxq("query layerservice vert.n ? selected") > 0)	{	&getVertPos;	}
		elsif (lxq("query layerservice vert.n ? selected") > 1)												{	&getVertPos;	}
		elsif (lxq("query layerservice edge.n ? selected") > 0)												{	&getEdgePos;	}
		else																								{	&die;			}
	}
	else{
		&die;
	}
}

if ($fullCut == 1){	&resizeCut;	}
lx("select.type polygon");
lx("tool.set poly.knife on");
lx("tool.reset");
if ( ($edgeAxis != 1) && ($worldAxis != 1) ){
	lx("tool.setAttr axis.view axis $axis");
	lx("tool.setAttr axis.view axisX {@axis[0]}");
	lx("tool.setAttr axis.view axisY {@axis[1]}");
	lx("tool.setAttr axis.view axisZ {@axis[2]}");
}else{
	lx("tool.setAttr axis.auto axis {$axis}");
}

lx("tool.setAttr poly.knife startX {@pos1[0]}");
lx("tool.setAttr poly.knife startY {@pos1[1]}");
lx("tool.setAttr poly.knife startZ {@pos1[2]}");
lx("tool.setAttr poly.knife endX {@pos2[0]}");
lx("tool.setAttr poly.knife endY {@pos2[1]}");
lx("tool.setAttr poly.knife endZ {@pos2[2]}");
lx("tool.doApply");
lx("tool.set poly.knife off");

if ($symmAxis != 3){
	lxout("[->] : Performing a symmetry slice");
	@axis[$symmAxis]*=-1;
	@pos1[$symmAxis]*=-1;
	@pos2[$symmAxis]*=-1;

	lx("tool.set poly.knife on");
	lx("tool.reset");
	if ($worldAxis != 1){
		lx("tool.setAttr axis.view axis $axis");
		lx("tool.setAttr axis.view axisX {@axis[0]}");
		lx("tool.setAttr axis.view axisY {@axis[1]}");
		lx("tool.setAttr axis.view axisZ {@axis[2]}");
	}else{
		lx("tool.setAttr axis.auto axis {$axis}");
	}
	lx("tool.setAttr poly.knife startX {@pos1[0]}");
	lx("tool.setAttr poly.knife startY {@pos1[1]}");
	lx("tool.setAttr poly.knife startZ {@pos1[2]}");
	lx("tool.setAttr poly.knife endX {@pos2[0]}");
	lx("tool.setAttr poly.knife endY {@pos2[1]}");
	lx("tool.setAttr poly.knife endZ {@pos2[2]}");
	lx("tool.doApply");
	lx("tool.set poly.knife off");
}




#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#------------[SCRIPT IS FINISHED] SAFETY REIMPLEMENTING-----------------
#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#reset workplane just to stop the layer reference from spinning the screen
lx("workplane.reset");

#Set the layer reference back
lx("!!layer.setReference [$layerReference]");

#put the WORKPLANE and UNIT MODE back to what you were in before.
lx("workPlane.edit {@WPmem[0]} {@WPmem[1]} {@WPmem[2]} {@WPmem[3]} {@WPmem[4]} {@WPmem[5]}");

#Set the action center settings back
if ($actr == 1) {	lx( "tool.set {$seltype} on" ); }
else { lx("tool.set center.$selCenter on"); lx("tool.set axis.$selAxis on"); }

#restore selection mode.
lx("select.type $selType");

#Set Symmetry back
if ($symmAxis != 3)
{
	#CONVERT MY OLDSCHOOL SYMM AXIS TO MODO's NEWSCHOOL NAME
	if 		($symmAxis == "3")	{	$symmAxis = "none";		}
	elsif	($symmAxis == "0")	{	$symmAxis = "x";		}
	elsif	($symmAxis == "1")	{	$symmAxis = "y";		}
	elsif	($symmAxis == "2")	{	$symmAxis = "z";		}
	lxout("turning symm back on ($symmAxis)"); lx("!!select.symmetryState {$symmAxis}");
}











#------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#-------------------------------------------------------------------------------------------------SUBROUTINES-------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#RESIZE CUT SUB
#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
sub resizeCut{
	lxout("[->] : Using 'fullCut' to perform a cut that will be as big as the current viewport.");
	my $scale = lxq("query view3dservice view.scale ? $viewport");
	my @dimensions = lxq("query view3dservice view.rect ? $viewport");
	my $diagDist = sqrt(@dimensions[2]*$scale*@dimensions[2]*$scale + @dimensions[3]*$scale*@dimensions[3]*$scale);
	my @posCenter = arrMath(arrMath(@pos1,@pos2,add),0.5,0.5,0.5,mult);
	my @disp1 = arrMath(@pos1,@posCenter,subt);
	my $dist = sqrt((@disp1[0]*@disp1[0])+(@disp1[1]*@disp1[1])+(@disp1[2]*@disp1[2]));
	my $mult = $diagDist/$dist;
	@pos1 = arrMath(@posCenter,arrMath(@disp1,$mult,$mult,$mult,mult),add);
	@pos2 = arrMath(@posCenter,arrMath(@disp1,-$mult,-$mult,-$mult,mult),add);
}


#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#DIE SUB
#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
sub die{
	die("\n.\n[----------------------You didn't have at least (2 verts) or (1 edge) selected so I'm killing the script.--------------------------]\n[--PLEASE TURN OFF THIS WARNING WINDOW by clicking on the (In the future) button and choose (Hide Message)--] \n[-----------------------------------This window is not supposed to come up, but I can't control that.---------------------------]\n.\n");
}

#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#GET VERT POSITIONS SUB
#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
sub getVertPos{
	lxout("[->] : VERT MODE : Using the first and last selected verts to determine the cut angle");
	my @verts = lxq("query layerservice verts ? selected");
	our @pos1 = lxq("query layerservice vert.pos ? @verts[0]");
	our @pos2 = lxq("query layerservice vert.pos ? @verts[-1]");

	#override the first vert Pos if horizontal or vertical is on. (don't think this'll work with screen axis)
	if ($vertical == 1){
		@pos1 = @pos2;
		if ($axis == 1)	{	@pos1[2] += 1;	}
		else			{	@pos1[1] += 1;	}
	}
	elsif ($horizontal == 1){
		@pos1 = @pos2;
		if ($axis == 0)	{	@pos1[2] += 1;	}
		else			{	@pos1[0] += 1;	}
	}
}

#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#GET EDGE POSITION SUB
#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
sub getEdgePos{
	lxout("[->] : EDGE MODE : Using the last selected edge to determine the cut angle");
	my @edges = lxq("query layerservice selection ? edge");
	@edges[-1] =~ s/\(\d{0,},/\(/;
	@edges[-1] =~ tr/()//d;
	my @verts = split(/,/, @edges[-1]);
	our @pos1 = lxq("query layerservice vert.pos ? @verts[0]");
	our @pos2 = lxq("query layerservice vert.pos ? @verts[1]");
}

#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#DOT PRODUCT subroutine
#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#USAGE : my $dp = dotProduct(\@vector1,\@vector2);

sub dotProduct{
	my @array1 = @{$_[0]};
	my @array2 = @{$_[1]};
	my $dp = (	(@array1[0]*@array2[0])+(@array1[1]*@array2[1])+(@array1[2]*@array2[2])	);
	return $dp;
}

#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#POPUP SUB
#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#USAGE : popup("What I wanna print");

sub popup #(MODO2 FIX)
{
	lx("dialog.setup yesNo");
	lx("dialog.msg {@_}");
	lx("dialog.open");
	my $confirm = lxq("dialog.result ?");
	if($confirm eq "no"){die;}
}

#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#PERFORM MATH FROM ONE ARRAY TO ANOTHER subroutine
#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#USAGE : my @disp = arrMath(@pos2,@pos1,subt);
sub arrMath{
	my @array1 = (@_[0],@_[1],@_[2]);
	my @array2 = (@_[3],@_[4],@_[5]);
	my $math = @_[6];

	my @newArray;
	if ($math eq "add")		{	@newArray = (@array1[0]+@array2[0],@array1[1]+@array2[1],@array1[2]+@array2[2]);	}
	elsif ($math eq "subt")	{	@newArray = (@array1[0]-@array2[0],@array1[1]-@array2[1],@array1[2]-@array2[2]);	}
	elsif ($math eq "mult")	{	@newArray = (@array1[0]*@array2[0],@array1[1]*@array2[1],@array1[2]*@array2[2]);	}
	elsif ($math eq "div")	{	@newArray = (@array1[0]/@array2[0],@array1[1]/@array2[1],@array1[2]/@array2[2]);	}
	return @newArray;
}


#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#UNIT VECTOR SUBROUTINE
#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#USAGE : my @unitVector = unitVector(@vector);
sub unitVector{
	my $dist1 = sqrt((@_[0]*@_[0])+(@_[1]*@_[1])+(@_[2]*@_[2]));
	@_ = ((@_[0]/$dist1),(@_[1]/$dist1),(@_[2]/$dist1));
	return @_;
}

#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#CONVERT MATRIX TO EULER (9char matrix)
#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#USAGE : my @rotations = matrixToEuler(\@vector1,\@vector2,\@vector3);
sub matrixToEuler{
	my @x = @{$_[0]};
	my @y = @{$_[1]};
	my @z = @{$_[2]};

	##TEMP : BUILD THE VERTS for the vector matrix
	#my @vert1 = arrMath(@x,30,30,30,mult);
	#my @vert2 = arrMath(@y,30,30,30,mult);
	#my @vert3 = arrMath(@z,30,30,30,mult);
	#@vert1 = arrMath(@objectBottom,@vert1,add);
	#@vert2 = arrMath(@objectBottom,@vert2,add);
	#@vert3 = arrMath(@objectBottom,@vert3,add);
	#lx("vert.new @objectBottom");
	#createSphere(@vert1);
	#createCube(@vert2);
	#lx("vert.new @vert3");

	my ($heading,$altitude,$bank);
	my $pi = 3.14159265358979323;

	if (@y[0] > 0.998){						#except when M10=1 (north pole)
		$heading = atan2(@x[2],@z[2]);		#heading = atan2(M02,M22)
		$altitude = asin(@y[0]);		 		#
		$bank = 0;							#bank = 0
	}elsif (@y[0] < -0.998){					#except when M10=-1 (south pole)
		$heading = atan2(@x[2],@z[2]);		#heading = atan2(M02,M22)
		$altitude = asin(@y[0]);				#
		$bank = 0;							#bank = 0
	}else{
		$heading = atan2(-@z[0],@x[0]);		#heading = atan2(-m20,m00)
		$altitude = asin(@y[0]);		  		#attitude = asin(m10)
		$bank = atan2(-@y[2],@y[1]);			#bank = atan2(-m12,m11)
	}

	return ($heading,$altitude,$bank);
}

#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#CROSSPRODUCT SUBROUTINE (in=4pos out=1vec)
#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#USAGE : my @crossProduct = crossProduct(\@vector1,\@vector2);
sub crossProduct{
	my @vector1 = @{$_[0]};
	my @vector2 = @{$_[1]};

	#create the crossproduct
	my @cp;
	@cp[0] = (@vector1[1]*@vector2[2])-(@vector2[1]*@vector1[2]);
	@cp[1] = (@vector1[2]*@vector2[0])-(@vector2[2]*@vector1[0]);
	@cp[2] = (@vector1[0]*@vector2[1])-(@vector2[0]*@vector1[1]);
	return @cp;
}

#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#ASIN subroutine (haven't tested it to make sure it works tho)
#------------------------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------------------------
#USAGE :
#my $ydeg =  &rad2deg(&asin($axis[1]/$yhyp));
sub asin{
	atan2($_[0], sqrt(1 - $_[0] * $_[0]));
}
