#perl
#BY: Seneca Menard
#version 1.1
#This script is to select all the touching polygons that have the same materials as the original selection..
#-(8-11-07 bugfix) : doesn't select hidden polys anymore.

#---------------------------------------------------------------------------------------------
#SETUP
#---------------------------------------------------------------------------------------------
my $mainlayer = lxq("query layerservice layers ? main");
my @originalPolys = lxq("query layerservice polys ? selected");
my @currentPolyList = @originalPolys;
my %ignorePolyList;
foreach my $poly (@originalPolys){ 	$ignorePolyList{$poly} = 1;	}
my @finalPolyList;
my %materialList;
my $loop = 1;
my $round = 1;

#---------------------------------------------------------------------------------------------
#BUILD THE MATERIAL HASH TABLE
#---------------------------------------------------------------------------------------------
foreach my $poly (@originalPolys){
	my $material = lxq("query layerservice poly.material ? $poly");
	$materialList{$material} = 1;
}


#---------------------------------------------------------------------------------------------
#LOOP THRU POLYS AND SELECT THE POLYS WITH SIMILAR MATERIAL.
#---------------------------------------------------------------------------------------------
while (@currentPolyList > 0){
	lxout("Round [$round]");

	#create and clean up all the variables that need to be created or cleaned.
	my %vertList;
	my %polyList;
	my @foundPolys;

	#go through all polys in the last list and find their verts.
	foreach my $poly (@currentPolyList){
		my @verts = lxq("query layerservice poly.vertList ? $poly");
		foreach my $vert (@verts){
			$vertList{$vert} = 1;
		}
	}

	#go through all the verts and find the polys (ignoring the ones that have already been checked.
	foreach my $vert (keys %vertList){
		my @polys = lxq("query layerservice vert.polyList ? $vert");
		foreach my $poly (@polys){
			if (lxq("query layerservice poly.hidden ? $poly") == 0){
				if ($ignorePolyList{$poly} == ""){
					$polyList{$poly} = 1;
				}
			}
		}
	}

	#go through all the "new" polys and check their materials.  add 'em to the list if they match
	foreach my $poly (keys %polyList){
		my $material = lxq("query layerservice poly.material ? $poly");
		if ($materialList{$material} == 1){
			push(@foundPolys,$poly);
		}

		#add all the polys to the ignore list for the next round
		$ignorePolyList{$poly} = 1;
	}

	#add the found poly list to the final poly list
	push (@finalPolyList, @foundPolys);

	#replace the currentPolyList with the found poly list so it's ready for next round.
	@currentPolyList = @foundPolys;

	#up the round count.
	$round++;
}



#---------------------------------------------------------------------------------------------
#SELECT ALL THE FOUND POLYS
#---------------------------------------------------------------------------------------------
foreach my $poly (@finalPolyList){
	lx("select.element $mainlayer polygon add $poly");
}


